import sys
import json
import html
from bs4 import BeautifulSoup

BOOKMARK_FILE_START_SEQUENCE = """
<!DOCTYPE NETSCAPE-Bookmark-file-1>
<!-- This is an automatically generated file.
     It will be read and overwritten.
     DO NOT EDIT! -->
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
<TITLE>Bookmarks</TITLE>
<H1>Bookmarks</H1>
<DL><p>
"""


def make_folder(h3_tag, children):
    res = {}
    res["title"] = h3_tag.text
    res["type"] = "folder"
    for key in ["add_date", "last_modified", "personal_toolbar_folder"]:
        res[key] = h3_tag.get(key)
    res["children"] = children
    return res


def make_item(a_tag):
    res = {}
    res["title"] = a_tag.text
    res["type"] = "item"
    for key in ["add_date", "bookmark", "href", "icon"]:
        res[key] = a_tag.get(key)
    return res


def process_node(node):
    if node.name == "dl":
        # Process folder
        return [process_node(child) for child in node.findChildren(recursive=False) if child.name == "dt"]  # noqa: E501
    if node.name == "dt":
        # Process folder OR link
        for child in node.findChildren(recursive=False):
            if child.name == "a":
                return make_item(child)
            if child.name == "h3":
                children = process_node(child.find_next_sibling("dl"))
                return make_folder(child, children)


def serialize_to_html(bookmarks):

    def fmt_tag_values(d, keys):
        res = ""
        for k in keys:
            if d[k]:
                res += ' %s="%s"' % (k.upper(), d[k])
        return res

    def _serialize_to_html(bookmarks, level):
        res = ""
        for bookmark in bookmarks:
            indent = " " * level * 4
            # Folder
            if bookmark["type"] == "folder":
                attrs = fmt_tag_values(bookmark, ["add_date", "last_modified",
                                                  "personal_toolbar_folder"])
                line = ("""<DT><H3%s>%s</H3>"""
                        % (attrs, html.escape(bookmark["title"])))
                res += indent + line + "\n"
                res += indent + "<DL><p>\n"
                res += _serialize_to_html(bookmark["children"], level+1)
                res += indent + "</DL><p>\n"
            # Item
            if bookmark["type"] == "item":
                line = ("""<DT><A%s>%s</A>""" %
                        (fmt_tag_values(bookmark, ["href", "add_date", "icon"]),
                         html.escape(bookmark["title"])))
                res += indent + line + "\n"
        return res

    res = BOOKMARK_FILE_START_SEQUENCE
    res += _serialize_to_html(bookmarks, 1)
    res += "</DL><p>"
    return res


def dump_chrome(bookmarks, f):
    f.write(serialize_to_html(bookmarks))


def load_chrome(bytes):
    soup = BeautifulSoup(bytes, "html5lib")
    bookmarks = process_node(soup.find_all("dl")[0])
    return bookmarks



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zbookmarks']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0']

setup_kwargs = {
    'name': 'zbookmarks',
    'version': '1.3',
    'description': 'Load / dump Chrome bookmark files',
    'long_description': '### Install\n\n```console\n$ pip install zbookmarks\n```\n\n### Usage\n\nA simple python package for loading / dumping Chrome HTML bookmarks.\nBorn out of necessity to fix my own bookmarks.\n\n```python\nimport zbookmarks\n\n# Load\nwith open("chrome_bookmarks.html", "r") as f:\n    bookmarks = zbookmarks.load_chrome(f.read())\n\n# Print\nprint(bookmarks)\n\n# Dump\nwith open("output.html", "w") as f:\n    zbookmarks.dump_chrome(bookmarks, f)\n```\n\n\n### How it works\n\nI made this package by examining my own Chrome bookmark files and extrapolating.\nThe general rules are:\n- `dl` denotes a list of bookmark items and folders.\n- `dt` denotes either a bookmark item or folder:\n    - If it\'s a bookmark item, `dt` has a single `a` tag child, which gives the\n      bookmark attributes (href, title, etc.).\n    - If it\'s a bookmark folder, `dt` has 3 children:\n        - A `h3` tag containing the folder attributes (title, etc.)\n        - A `dl` tag containing the folder\'s contents.\n        - A useless `p` child tag.\n\nI found it easier to come up with these rules after visualizing the DOM tree of\na Chrome bookmarks file:\n\n![Visualize DOM tree of bookmarks file](scripts/dom.svg)\n\nYou can try it out on your own bookmark file by running:\n```console\n# You need to install graphviz (dot) in order to run the script\n$ sudo apt-get install -y graphviz\n$ poetry install\n$ python3 scripts/visualize_dom.py <my_bookmarks_file.html>\n```\n',
    'author': 'Yannis Zarkadas',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
